@@ -1,5 +1,6 @@
 import pytest
 from _pytest._io.saferepr import _pformat_dispatch
+from _pytest._io.saferepr import DEFAULT_REPR_MAX_SIZE
 from _pytest._io.saferepr import saferepr
 
 
@@ -15,6 +16,13 @@ def test_maxsize():
     assert s == expected
 
 
+def test_no_maxsize():
+    text = "x" * DEFAULT_REPR_MAX_SIZE * 10
+    s = saferepr(text, maxsize=None)
+    expected = repr(text)
+    assert s == expected
+
+
 def test_maxsize_error_on_instance():
     class A:
         def __repr__(self):
@@ -25,7 +33,7 @@ def test_maxsize_error_on_instance():
     assert s[0] == "(" and s[-1] == ")"
 
 
-def test_exceptions():
+def test_exceptions() -> None:
     class BrokenRepr:
         def __init__(self, ex):
             self.ex = ex
@@ -34,8 +42,8 @@ def test_exceptions():
             raise self.ex
 
     class BrokenReprException(Exception):
-        __str__ = None
-        __repr__ = None
+        __str__ = None  # type: ignore[assignment]
+        __repr__ = None  # type: ignore[assignment]
 
     assert "Exception" in saferepr(BrokenRepr(Exception("broken")))
     s = saferepr(BrokenReprException("really broken"))
@@ -44,7 +52,7 @@ def test_exceptions():
 
     none = None
     try:
-        none()
+        none()  # type: ignore[misc]
     except BaseException as exc:
         exp_exc = repr(exc)
     obj = BrokenRepr(BrokenReprException("omg even worse"))
@@ -136,10 +144,10 @@ def test_big_repr():
     assert len(saferepr(range(1000))) <= len("[" + SafeRepr(0).maxlist * "1000" + "]")
 
 
-def test_repr_on_newstyle():
+def test_repr_on_newstyle() -> None:
     class Function:
         def __repr__(self):
-            return "<%s>" % (self.name)
+            return "<%s>" % (self.name)  # type: ignore[attr-defined]
 
     assert saferepr(Function())
 
