@@ -2,6 +2,7 @@ import enum
 import sys
 from functools import partial
 from functools import wraps
+from typing import TYPE_CHECKING
 from typing import Union
 
 import pytest
@@ -12,14 +13,14 @@ from _pytest.compat import get_real_func
 from _pytest.compat import is_generator
 from _pytest.compat import safe_getattr
 from _pytest.compat import safe_isclass
-from _pytest.compat import TYPE_CHECKING
 from _pytest.outcomes import OutcomeException
+from _pytest.pytester import Pytester
 
 if TYPE_CHECKING:
     from typing_extensions import Literal
 
 
-def test_is_generator():
+def test_is_generator() -> None:
     def zap():
         yield  # pragma: no cover
 
@@ -30,13 +31,13 @@ def test_is_generator():
     assert not is_generator(foo)
 
 
-def test_real_func_loop_limit():
+def test_real_func_loop_limit() -> None:
     class Evil:
         def __init__(self):
             self.left = 1000
 
         def __repr__(self):
-            return "<Evil left={left}>".format(left=self.left)
+            return f"<Evil left={self.left}>"
 
         def __getattr__(self, attr):
             if not self.left:
@@ -56,7 +57,7 @@ def test_real_func_loop_limit():
         get_real_func(evil)
 
 
-def test_get_real_func():
+def test_get_real_func() -> None:
     """Check that get_real_func correctly unwraps decorators until reaching the real function"""
 
     def decorator(f):
@@ -81,7 +82,7 @@ def test_get_real_func():
     assert get_real_func(wrapped_func2) is wrapped_func
 
 
-def test_get_real_func_partial():
+def test_get_real_func_partial() -> None:
     """Test get_real_func handles partial instances correctly"""
 
     def foo(x):
@@ -91,8 +92,9 @@ def test_get_real_func_partial():
     assert get_real_func(partial(foo)) is foo
 
 
-def test_is_generator_asyncio(testdir):
-    testdir.makepyfile(
+@pytest.mark.skipif(sys.version_info >= (3, 11), reason="couroutine removed")
+def test_is_generator_asyncio(pytester: Pytester) -> None:
+    pytester.makepyfile(
         """
         from _pytest.compat import is_generator
         import asyncio
@@ -106,12 +108,12 @@ def test_is_generator_asyncio(testdir):
     )
     # avoid importing asyncio into pytest's own process,
     # which in turn imports logging (#8)
-    result = testdir.runpytest_subprocess()
+    result = pytester.runpytest_subprocess()
     result.stdout.fnmatch_lines(["*1 passed*"])
 
 
-def test_is_generator_async_syntax(testdir):
-    testdir.makepyfile(
+def test_is_generator_async_syntax(pytester: Pytester) -> None:
+    pytester.makepyfile(
         """
         from _pytest.compat import is_generator
         def test_is_generator_py35():
@@ -125,18 +127,15 @@ def test_is_generator_async_syntax(testdir):
             assert not is_generator(bar)
     """
     )
-    result = testdir.runpytest()
+    result = pytester.runpytest()
     result.stdout.fnmatch_lines(["*1 passed*"])
 
 
-@pytest.mark.skipif(
-    sys.version_info < (3, 6), reason="async gen syntax available in Python 3.6+"
-)
-def test_is_generator_async_gen_syntax(testdir):
-    testdir.makepyfile(
+def test_is_generator_async_gen_syntax(pytester: Pytester) -> None:
+    pytester.makepyfile(
         """
         from _pytest.compat import is_generator
-        def test_is_generator_py36():
+        def test_is_generator():
             async def foo():
                 yield
                 await foo()
@@ -148,7 +147,7 @@ def test_is_generator_async_gen_syntax(testdir):
             assert not is_generator(bar)
     """
     )
-    result = testdir.runpytest()
+    result = pytester.runpytest()
     result.stdout.fnmatch_lines(["*1 passed*"])
 
 
@@ -159,14 +158,14 @@ class ErrorsHelper:
 
     @property
     def raise_exception(self):
-        raise Exception("exception should be catched")
+        raise Exception("exception should be caught")
 
     @property
     def raise_fail_outcome(self):
-        pytest.fail("fail should be catched")
+        pytest.fail("fail should be caught")
 
 
-def test_helper_failures():
+def test_helper_failures() -> None:
     helper = ErrorsHelper()
     with pytest.raises(Exception):
         helper.raise_exception
@@ -174,7 +173,7 @@ def test_helper_failures():
         helper.raise_fail_outcome
 
 
-def test_safe_getattr():
+def test_safe_getattr() -> None:
     helper = ErrorsHelper()
     assert safe_getattr(helper, "raise_exception", "default") == "default"
     assert safe_getattr(helper, "raise_fail_outcome", "default") == "default"
@@ -182,7 +181,7 @@ def test_safe_getattr():
         assert safe_getattr(helper, "raise_baseexception", "default")
 
 
-def test_safe_isclass():
+def test_safe_isclass() -> None:
     assert safe_isclass(type) is True
 
     class CrappyClass(Exception):
@@ -215,7 +214,7 @@ def test_cached_property() -> None:
 
 
 def test_assert_never_union() -> None:
-    x = 10  # type: Union[int, str]
+    x: Union[int, str] = 10
 
     if isinstance(x, int):
         pass
@@ -233,7 +232,7 @@ def test_assert_never_union() -> None:
 
 def test_assert_never_enum() -> None:
     E = enum.Enum("E", "a b")
-    x = E.a  # type: E
+    x: E = E.a
 
     if x is E.a:
         pass
@@ -250,7 +249,7 @@ def test_assert_never_enum() -> None:
 
 
 def test_assert_never_literal() -> None:
-    x = "a"  # type: Literal["a", "b"]
+    x: Literal["a", "b"] = "a"
 
     if x == "a":
         pass
