@@ -1,17 +1,12 @@
 import os
 import sys
+from typing import List
 
 import _pytest._code
 import pytest
 from _pytest.debugging import _validate_usepdb_cls
-
-try:
-    # Type ignored for Python <= 3.6.
-    breakpoint  # type: ignore
-except NameError:
-    SUPPORTS_BREAKPOINT_BUILTIN = False
-else:
-    SUPPORTS_BREAKPOINT_BUILTIN = True
+from _pytest.monkeypatch import MonkeyPatch
+from _pytest.pytester import Pytester
 
 
 _ENVIRON_PYTHONBREAKPOINT = os.environ.get("PYTHONBREAKPOINT", "")
@@ -19,22 +14,22 @@ _ENVIRON_PYTHONBREAKPOINT = os.environ.get("PYTHONBREAKPOINT", "")
 
 @pytest.fixture(autouse=True)
 def pdb_env(request):
-    if "testdir" in request.fixturenames:
+    if "pytester" in request.fixturenames:
         # Disable pdb++ with inner tests.
-        testdir = request.getfixturevalue("testdir")
-        testdir.monkeypatch.setenv("PDBPP_HIJACK_PDB", "0")
+        pytester = request.getfixturevalue("pytester")
+        pytester._monkeypatch.setenv("PDBPP_HIJACK_PDB", "0")
 
 
-def runpdb_and_get_report(testdir, source):
-    p = testdir.makepyfile(source)
-    result = testdir.runpytest_inprocess("--pdb", p)
-    reports = result.reprec.getreports("pytest_runtest_logreport")
+def runpdb_and_get_report(pytester: Pytester, source: str):
+    p = pytester.makepyfile(source)
+    result = pytester.runpytest_inprocess("--pdb", p)
+    reports = result.reprec.getreports("pytest_runtest_logreport")  # type: ignore[attr-defined]
     assert len(reports) == 3, reports  # setup/call/teardown
     return reports[1]
 
 
 @pytest.fixture
-def custom_pdb_calls():
+def custom_pdb_calls() -> List[str]:
     called = []
 
     # install dummy debugger class and track which methods were called on it
@@ -91,9 +86,9 @@ class TestPDB:
         monkeypatch.setattr(plugin, "post_mortem", mypdb)
         return pdblist
 
-    def test_pdb_on_fail(self, testdir, pdblist):
+    def test_pdb_on_fail(self, pytester: Pytester, pdblist) -> None:
         rep = runpdb_and_get_report(
-            testdir,
+            pytester,
             """
             def test_func():
                 assert 0
@@ -104,9 +99,9 @@ class TestPDB:
         tb = _pytest._code.Traceback(pdblist[0][0])
         assert tb[-1].name == "test_func"
 
-    def test_pdb_on_xfail(self, testdir, pdblist):
+    def test_pdb_on_xfail(self, pytester: Pytester, pdblist) -> None:
         rep = runpdb_and_get_report(
-            testdir,
+            pytester,
             """
             import pytest
             @pytest.mark.xfail
@@ -117,9 +112,9 @@ class TestPDB:
         assert "xfail" in rep.keywords
         assert not pdblist
 
-    def test_pdb_on_skip(self, testdir, pdblist):
+    def test_pdb_on_skip(self, pytester, pdblist) -> None:
         rep = runpdb_and_get_report(
-            testdir,
+            pytester,
             """
             import pytest
             def test_func():
@@ -129,9 +124,9 @@ class TestPDB:
         assert rep.skipped
         assert len(pdblist) == 0
 
-    def test_pdb_on_BdbQuit(self, testdir, pdblist):
+    def test_pdb_on_BdbQuit(self, pytester, pdblist) -> None:
         rep = runpdb_and_get_report(
-            testdir,
+            pytester,
             """
             import bdb
             def test_func():
@@ -141,9 +136,9 @@ class TestPDB:
         assert rep.failed
         assert len(pdblist) == 0
 
-    def test_pdb_on_KeyboardInterrupt(self, testdir, pdblist):
+    def test_pdb_on_KeyboardInterrupt(self, pytester, pdblist) -> None:
         rep = runpdb_and_get_report(
-            testdir,
+            pytester,
             """
             def test_func():
                 raise KeyboardInterrupt
@@ -160,8 +155,8 @@ class TestPDB:
             child.wait()
         assert not child.isalive()
 
-    def test_pdb_unittest_postmortem(self, testdir):
-        p1 = testdir.makepyfile(
+    def test_pdb_unittest_postmortem(self, pytester: Pytester) -> None:
+        p1 = pytester.makepyfile(
             """
             import unittest
             class Blub(unittest.TestCase):
@@ -172,7 +167,7 @@ class TestPDB:
                     assert 0
         """
         )
-        child = testdir.spawn_pytest("--pdb %s" % p1)
+        child = pytester.spawn_pytest(f"--pdb {p1}")
         child.expect("Pdb")
         child.sendline("p self.filename")
         child.sendeof()
@@ -180,9 +175,9 @@ class TestPDB:
         assert "debug.me" in rest
         self.flush(child)
 
-    def test_pdb_unittest_skip(self, testdir):
+    def test_pdb_unittest_skip(self, pytester: Pytester) -> None:
         """Test for issue #2137"""
-        p1 = testdir.makepyfile(
+        p1 = pytester.makepyfile(
             """
             import unittest
             @unittest.skipIf(True, 'Skipping also with pdb active')
@@ -191,14 +186,14 @@ class TestPDB:
                     assert 0
         """
         )
-        child = testdir.spawn_pytest("-rs --pdb %s" % p1)
+        child = pytester.spawn_pytest(f"-rs --pdb {p1}")
         child.expect("Skipping also with pdb active")
         child.expect_exact("= 1 skipped in")
         child.sendeof()
         self.flush(child)
 
-    def test_pdb_print_captured_stdout_and_stderr(self, testdir):
-        p1 = testdir.makepyfile(
+    def test_pdb_print_captured_stdout_and_stderr(self, pytester: Pytester) -> None:
+        p1 = pytester.makepyfile(
             """
             def test_1():
                 import sys
@@ -210,7 +205,7 @@ class TestPDB:
                 pass
         """
         )
-        child = testdir.spawn_pytest("--pdb %s" % p1)
+        child = pytester.spawn_pytest("--pdb %s" % p1)
         child.expect("captured stdout")
         child.expect("get rekt")
         child.expect("captured stderr")
@@ -226,14 +221,16 @@ class TestPDB:
         assert "get rekt" not in rest
         self.flush(child)
 
-    def test_pdb_dont_print_empty_captured_stdout_and_stderr(self, testdir):
-        p1 = testdir.makepyfile(
+    def test_pdb_dont_print_empty_captured_stdout_and_stderr(
+        self, pytester: Pytester
+    ) -> None:
+        p1 = pytester.makepyfile(
             """
             def test_1():
                 assert False
         """
         )
-        child = testdir.spawn_pytest("--pdb %s" % p1)
+        child = pytester.spawn_pytest("--pdb %s" % p1)
         child.expect("Pdb")
         output = child.before.decode("utf8")
         child.sendeof()
@@ -242,8 +239,8 @@ class TestPDB:
         self.flush(child)
 
     @pytest.mark.parametrize("showcapture", ["all", "no", "log"])
-    def test_pdb_print_captured_logs(self, testdir, showcapture):
-        p1 = testdir.makepyfile(
+    def test_pdb_print_captured_logs(self, pytester, showcapture: str) -> None:
+        p1 = pytester.makepyfile(
             """
             def test_1():
                 import logging
@@ -251,9 +248,7 @@ class TestPDB:
                 assert False
         """
         )
-        child = testdir.spawn_pytest(
-            "--show-capture={} --pdb {}".format(showcapture, p1)
-        )
+        child = pytester.spawn_pytest(f"--show-capture={showcapture} --pdb {p1}")
         if showcapture in ("all", "log"):
             child.expect("captured log")
             child.expect("get rekt")
@@ -263,8 +258,8 @@ class TestPDB:
         assert "1 failed" in rest
         self.flush(child)
 
-    def test_pdb_print_captured_logs_nologging(self, testdir):
-        p1 = testdir.makepyfile(
+    def test_pdb_print_captured_logs_nologging(self, pytester: Pytester) -> None:
+        p1 = pytester.makepyfile(
             """
             def test_1():
                 import logging
@@ -272,7 +267,7 @@ class TestPDB:
                 assert False
         """
         )
-        child = testdir.spawn_pytest("--show-capture=all --pdb -p no:logging %s" % p1)
+        child = pytester.spawn_pytest("--show-capture=all --pdb -p no:logging %s" % p1)
         child.expect("get rekt")
         output = child.before.decode("utf8")
         assert "captured log" not in output
@@ -282,8 +277,8 @@ class TestPDB:
         assert "1 failed" in rest
         self.flush(child)
 
-    def test_pdb_interaction_exception(self, testdir):
-        p1 = testdir.makepyfile(
+    def test_pdb_interaction_exception(self, pytester: Pytester) -> None:
+        p1 = pytester.makepyfile(
             """
             import pytest
             def globalfunc():
@@ -292,7 +287,7 @@ class TestPDB:
                 pytest.raises(ValueError, globalfunc)
         """
         )
-        child = testdir.spawn_pytest("--pdb %s" % p1)
+        child = pytester.spawn_pytest("--pdb %s" % p1)
         child.expect(".*def test_1")
         child.expect(".*pytest.raises.*globalfunc")
         child.expect("Pdb")
@@ -302,29 +297,29 @@ class TestPDB:
         child.expect("1 failed")
         self.flush(child)
 
-    def test_pdb_interaction_on_collection_issue181(self, testdir):
-        p1 = testdir.makepyfile(
+    def test_pdb_interaction_on_collection_issue181(self, pytester: Pytester) -> None:
+        p1 = pytester.makepyfile(
             """
             import pytest
             xxx
         """
         )
-        child = testdir.spawn_pytest("--pdb %s" % p1)
+        child = pytester.spawn_pytest("--pdb %s" % p1)
         # child.expect(".*import pytest.*")
         child.expect("Pdb")
         child.sendline("c")
         child.expect("1 error")
         self.flush(child)
 
-    def test_pdb_interaction_on_internal_error(self, testdir):
-        testdir.makeconftest(
+    def test_pdb_interaction_on_internal_error(self, pytester: Pytester) -> None:
+        pytester.makeconftest(
             """
             def pytest_runtest_protocol():
                 0/0
         """
         )
-        p1 = testdir.makepyfile("def test_func(): pass")
-        child = testdir.spawn_pytest("--pdb %s" % p1)
+        p1 = pytester.makepyfile("def test_func(): pass")
+        child = pytester.spawn_pytest("--pdb %s" % p1)
         child.expect("Pdb")
 
         # INTERNALERROR is only displayed once via terminal reporter.
@@ -342,17 +337,24 @@ class TestPDB:
         child.sendeof()
         self.flush(child)
 
-    def test_pdb_prevent_ConftestImportFailure_hiding_exception(self, testdir):
-        testdir.makepyfile("def test_func(): pass")
-        sub_dir = testdir.tmpdir.join("ns").ensure_dir()
-        sub_dir.join("conftest").new(ext=".py").write("import unknown")
-        sub_dir.join("test_file").new(ext=".py").write("def test_func(): pass")
+    def test_pdb_prevent_ConftestImportFailure_hiding_exception(
+        self, pytester: Pytester
+    ) -> None:
+        pytester.makepyfile("def test_func(): pass")
+        sub_dir = pytester.path.joinpath("ns")
+        sub_dir.mkdir()
+        sub_dir.joinpath("conftest").with_suffix(".py").write_text(
+            "import unknown", "utf-8"
+        )
+        sub_dir.joinpath("test_file").with_suffix(".py").write_text(
+            "def test_func(): pass", "utf-8"
+        )
 
-        result = testdir.runpytest_subprocess("--pdb", ".")
+        result = pytester.runpytest_subprocess("--pdb", ".")
         result.stdout.fnmatch_lines(["-> import unknown"])
 
-    def test_pdb_interaction_capturing_simple(self, testdir):
-        p1 = testdir.makepyfile(
+    def test_pdb_interaction_capturing_simple(self, pytester: Pytester) -> None:
+        p1 = pytester.makepyfile(
             """
             import pytest
             def test_1():
@@ -363,7 +365,7 @@ class TestPDB:
                 assert 0
         """
         )
-        child = testdir.spawn_pytest(str(p1))
+        child = pytester.spawn_pytest(str(p1))
         child.expect(r"test_1\(\)")
         child.expect("i == 1")
         child.expect("Pdb")
@@ -375,8 +377,8 @@ class TestPDB:
         assert "hello17" in rest  # out is captured
         self.flush(child)
 
-    def test_pdb_set_trace_kwargs(self, testdir):
-        p1 = testdir.makepyfile(
+    def test_pdb_set_trace_kwargs(self, pytester: Pytester) -> None:
+        p1 = pytester.makepyfile(
             """
             import pytest
             def test_1():
@@ -387,7 +389,7 @@ class TestPDB:
                 assert 0
         """
         )
-        child = testdir.spawn_pytest(str(p1))
+        child = pytester.spawn_pytest(str(p1))
         child.expect("== my_header ==")
         assert "PDB set_trace" not in child.before.decode()
         child.expect("Pdb")
@@ -398,15 +400,15 @@ class TestPDB:
         assert "hello17" in rest  # out is captured
         self.flush(child)
 
-    def test_pdb_set_trace_interception(self, testdir):
-        p1 = testdir.makepyfile(
+    def test_pdb_set_trace_interception(self, pytester: Pytester) -> None:
+        p1 = pytester.makepyfile(
             """
             import pdb
             def test_1():
                 pdb.set_trace()
         """
         )
-        child = testdir.spawn_pytest(str(p1))
+        child = pytester.spawn_pytest(str(p1))
         child.expect("test_1")
         child.expect("Pdb")
         child.sendline("q")
@@ -416,8 +418,8 @@ class TestPDB:
         assert "BdbQuit" not in rest
         self.flush(child)
 
-    def test_pdb_and_capsys(self, testdir):
-        p1 = testdir.makepyfile(
+    def test_pdb_and_capsys(self, pytester: Pytester) -> None:
+        p1 = pytester.makepyfile(
             """
             import pytest
             def test_1(capsys):
@@ -425,7 +427,7 @@ class TestPDB:
                 pytest.set_trace()
         """
         )
-        child = testdir.spawn_pytest(str(p1))
+        child = pytester.spawn_pytest(str(p1))
         child.expect("test_1")
         child.send("capsys.readouterr()\n")
         child.expect("hello1")
@@ -433,8 +435,8 @@ class TestPDB:
         child.read()
         self.flush(child)
 
-    def test_pdb_with_caplog_on_pdb_invocation(self, testdir):
-        p1 = testdir.makepyfile(
+    def test_pdb_with_caplog_on_pdb_invocation(self, pytester: Pytester) -> None:
+        p1 = pytester.makepyfile(
             """
             def test_1(capsys, caplog):
                 import logging
@@ -442,7 +444,7 @@ class TestPDB:
                 assert 0
         """
         )
-        child = testdir.spawn_pytest("--pdb %s" % str(p1))
+        child = pytester.spawn_pytest("--pdb %s" % str(p1))
         child.send("caplog.record_tuples\n")
         child.expect_exact(
             "[('test_pdb_with_caplog_on_pdb_invocation', 30, 'some_warning')]"
@@ -451,8 +453,8 @@ class TestPDB:
         child.read()
         self.flush(child)
 
-    def test_set_trace_capturing_afterwards(self, testdir):
-        p1 = testdir.makepyfile(
+    def test_set_trace_capturing_afterwards(self, pytester: Pytester) -> None:
+        p1 = pytester.makepyfile(
             """
             import pdb
             def test_1():
@@ -462,7 +464,7 @@ class TestPDB:
                 assert 0
         """
         )
-        child = testdir.spawn_pytest(str(p1))
+        child = pytester.spawn_pytest(str(p1))
         child.expect("test_1")
         child.send("c\n")
         child.expect("test_2")
@@ -472,8 +474,8 @@ class TestPDB:
         child.read()
         self.flush(child)
 
-    def test_pdb_interaction_doctest(self, testdir):
-        p1 = testdir.makepyfile(
+    def test_pdb_interaction_doctest(self, pytester: Pytester) -> None:
+        p1 = pytester.makepyfile(
             """
             def function_1():
                 '''
@@ -482,7 +484,7 @@ class TestPDB:
                 '''
         """
         )
-        child = testdir.spawn_pytest("--doctest-modules --pdb %s" % p1)
+        child = pytester.spawn_pytest("--doctest-modules --pdb %s" % p1)
         child.expect("Pdb")
 
         assert "UNEXPECTED EXCEPTION: AssertionError()" in child.before.decode("utf8")
@@ -498,8 +500,8 @@ class TestPDB:
         assert "1 failed" in rest
         self.flush(child)
 
-    def test_doctest_set_trace_quit(self, testdir):
-        p1 = testdir.makepyfile(
+    def test_doctest_set_trace_quit(self, pytester: Pytester) -> None:
+        p1 = pytester.makepyfile(
             """
             def function_1():
                 '''
@@ -509,7 +511,7 @@ class TestPDB:
         )
         # NOTE: does not use pytest.set_trace, but Python's patched pdb,
         #       therefore "-s" is required.
-        child = testdir.spawn_pytest("--doctest-modules --pdb -s %s" % p1)
+        child = pytester.spawn_pytest("--doctest-modules --pdb -s %s" % p1)
         child.expect("Pdb")
         child.sendline("q")
         rest = child.read().decode("utf8")
@@ -519,8 +521,8 @@ class TestPDB:
         assert "BdbQuit" not in rest
         assert "UNEXPECTED EXCEPTION" not in rest
 
-    def test_pdb_interaction_capturing_twice(self, testdir):
-        p1 = testdir.makepyfile(
+    def test_pdb_interaction_capturing_twice(self, pytester: Pytester) -> None:
+        p1 = pytester.makepyfile(
             """
             import pytest
             def test_1():
@@ -534,7 +536,7 @@ class TestPDB:
                 assert 0
         """
         )
-        child = testdir.spawn_pytest(str(p1))
+        child = pytester.spawn_pytest(str(p1))
         child.expect(r"PDB set_trace \(IO-capturing turned off\)")
         child.expect("test_1")
         child.expect("x = 3")
@@ -554,11 +556,11 @@ class TestPDB:
         assert "1 failed" in rest
         self.flush(child)
 
-    def test_pdb_with_injected_do_debug(self, testdir):
+    def test_pdb_with_injected_do_debug(self, pytester: Pytester) -> None:
         """Simulates pdbpp, which injects Pdb into do_debug, and uses
         self.__class__ in do_continue.
         """
-        p1 = testdir.makepyfile(
+        p1 = pytester.makepyfile(
             mytest="""
             import pdb
             import pytest
@@ -600,7 +602,7 @@ class TestPDB:
                 pytest.fail("expected_failure")
         """
         )
-        child = testdir.spawn_pytest("--pdbcls=mytest:CustomPdb %s" % str(p1))
+        child = pytester.spawn_pytest("--pdbcls=mytest:CustomPdb %s" % str(p1))
         child.expect(r"PDB set_trace \(IO-capturing turned off\)")
         child.expect(r"\n\(Pdb")
         child.sendline("debug foo()")
@@ -629,15 +631,15 @@ class TestPDB:
         assert "AssertionError: unexpected_failure" not in rest
         self.flush(child)
 
-    def test_pdb_without_capture(self, testdir):
-        p1 = testdir.makepyfile(
+    def test_pdb_without_capture(self, pytester: Pytester) -> None:
+        p1 = pytester.makepyfile(
             """
             import pytest
             def test_1():
                 pytest.set_trace()
         """
         )
-        child = testdir.spawn_pytest("-s %s" % p1)
+        child = pytester.spawn_pytest("-s %s" % p1)
         child.expect(r">>> PDB set_trace >>>")
         child.expect("Pdb")
         child.sendline("c")
@@ -646,13 +648,15 @@ class TestPDB:
         self.flush(child)
 
     @pytest.mark.parametrize("capture_arg", ("", "-s", "-p no:capture"))
-    def test_pdb_continue_with_recursive_debug(self, capture_arg, testdir):
+    def test_pdb_continue_with_recursive_debug(
+        self, capture_arg, pytester: Pytester
+    ) -> None:
         """Full coverage for do_debug without capturing.
 
         This is very similar to test_pdb_interaction_continue_recursive in general,
         but mocks out ``pdb.set_trace`` for providing more coverage.
         """
-        p1 = testdir.makepyfile(
+        p1 = pytester.makepyfile(
             """
             try:
                 input = raw_input
@@ -706,7 +710,7 @@ class TestPDB:
                 set_trace()
         """
         )
-        child = testdir.spawn_pytest("--tb=short {} {}".format(p1, capture_arg))
+        child = pytester.spawn_pytest(f"--tb=short {p1} {capture_arg}")
         child.expect("=== SET_TRACE ===")
         before = child.before.decode("utf8")
         if not capture_arg:
@@ -736,22 +740,22 @@ class TestPDB:
             assert "> PDB continue >" in rest
         assert "= 1 passed in" in rest
 
-    def test_pdb_used_outside_test(self, testdir):
-        p1 = testdir.makepyfile(
+    def test_pdb_used_outside_test(self, pytester: Pytester) -> None:
+        p1 = pytester.makepyfile(
             """
             import pytest
             pytest.set_trace()
             x = 5
         """
         )
-        child = testdir.spawn("{} {}".format(sys.executable, p1))
+        child = pytester.spawn(f"{sys.executable} {p1}")
         child.expect("x = 5")
         child.expect("Pdb")
         child.sendeof()
         self.flush(child)
 
-    def test_pdb_used_in_generate_tests(self, testdir):
-        p1 = testdir.makepyfile(
+    def test_pdb_used_in_generate_tests(self, pytester: Pytester) -> None:
+        p1 = pytester.makepyfile(
             """
             import pytest
             def pytest_generate_tests(metafunc):
@@ -761,22 +765,24 @@ class TestPDB:
                 pass
         """
         )
-        child = testdir.spawn_pytest(str(p1))
+        child = pytester.spawn_pytest(str(p1))
         child.expect("x = 5")
         child.expect("Pdb")
         child.sendeof()
         self.flush(child)
 
-    def test_pdb_collection_failure_is_shown(self, testdir):
-        p1 = testdir.makepyfile("xxx")
-        result = testdir.runpytest_subprocess("--pdb", p1)
+    def test_pdb_collection_failure_is_shown(self, pytester: Pytester) -> None:
+        p1 = pytester.makepyfile("xxx")
+        result = pytester.runpytest_subprocess("--pdb", p1)
         result.stdout.fnmatch_lines(
             ["E   NameError: *xxx*", "*! *Exit: Quitting debugger !*"]  # due to EOF
         )
 
     @pytest.mark.parametrize("post_mortem", (False, True))
-    def test_enter_leave_pdb_hooks_are_called(self, post_mortem, testdir):
-        testdir.makeconftest(
+    def test_enter_leave_pdb_hooks_are_called(
+        self, post_mortem, pytester: Pytester
+    ) -> None:
+        pytester.makeconftest(
             """
             mypdb = None
 
@@ -800,7 +806,7 @@ class TestPDB:
                 assert mypdb.set_attribute == "bar"
         """
         )
-        p1 = testdir.makepyfile(
+        p1 = pytester.makepyfile(
             """
             import pytest
 
@@ -813,9 +819,9 @@ class TestPDB:
         """
         )
         if post_mortem:
-            child = testdir.spawn_pytest(str(p1) + " --pdb -s -k test_post_mortem")
+            child = pytester.spawn_pytest(str(p1) + " --pdb -s -k test_post_mortem")
         else:
-            child = testdir.spawn_pytest(str(p1) + " -k test_set_trace")
+            child = pytester.spawn_pytest(str(p1) + " -k test_set_trace")
         child.expect("enter_pdb_hook")
         child.sendline("c")
         if post_mortem:
@@ -828,14 +834,18 @@ class TestPDB:
         assert "1 failed" in rest
         self.flush(child)
 
-    def test_pdb_custom_cls(self, testdir, custom_pdb_calls):
-        p1 = testdir.makepyfile("""xxx """)
-        result = testdir.runpytest_inprocess("--pdb", "--pdbcls=_pytest:_CustomPdb", p1)
+    def test_pdb_custom_cls(
+        self, pytester: Pytester, custom_pdb_calls: List[str]
+    ) -> None:
+        p1 = pytester.makepyfile("""xxx """)
+        result = pytester.runpytest_inprocess(
+            "--pdb", "--pdbcls=_pytest:_CustomPdb", p1
+        )
         result.stdout.fnmatch_lines(["*NameError*xxx*", "*1 error*"])
         assert custom_pdb_calls == ["init", "reset", "interaction"]
 
-    def test_pdb_custom_cls_invalid(self, testdir):
-        result = testdir.runpytest_inprocess("--pdbcls=invalid")
+    def test_pdb_custom_cls_invalid(self, pytester: Pytester) -> None:
+        result = pytester.runpytest_inprocess("--pdbcls=invalid")
         result.stderr.fnmatch_lines(
             [
                 "*: error: argument --pdbcls: 'invalid' is not in the format 'modname:classname'"
@@ -850,14 +860,20 @@ class TestPDB:
 
         assert _validate_usepdb_cls("pdb:DoesNotExist") == ("pdb", "DoesNotExist")
 
-    def test_pdb_custom_cls_without_pdb(self, testdir, custom_pdb_calls):
-        p1 = testdir.makepyfile("""xxx """)
-        result = testdir.runpytest_inprocess("--pdbcls=_pytest:_CustomPdb", p1)
+    def test_pdb_custom_cls_without_pdb(
+        self, pytester: Pytester, custom_pdb_calls: List[str]
+    ) -> None:
+        p1 = pytester.makepyfile("""xxx """)
+        result = pytester.runpytest_inprocess("--pdbcls=_pytest:_CustomPdb", p1)
         result.stdout.fnmatch_lines(["*NameError*xxx*", "*1 error*"])
         assert custom_pdb_calls == []
 
-    def test_pdb_custom_cls_with_set_trace(self, testdir, monkeypatch):
-        testdir.makepyfile(
+    def test_pdb_custom_cls_with_set_trace(
+        self,
+        pytester: Pytester,
+        monkeypatch: MonkeyPatch,
+    ) -> None:
+        pytester.makepyfile(
             custom_pdb="""
             class CustomPdb(object):
                 def __init__(self, *args, **kwargs):
@@ -870,7 +886,7 @@ class TestPDB:
                     print('custom set_trace>')
          """
         )
-        p1 = testdir.makepyfile(
+        p1 = pytester.makepyfile(
             """
             import pytest
 
@@ -878,8 +894,8 @@ class TestPDB:
                 pytest.set_trace(skip=['foo.*'])
         """
         )
-        monkeypatch.setenv("PYTHONPATH", str(testdir.tmpdir))
-        child = testdir.spawn_pytest("--pdbcls=custom_pdb:CustomPdb %s" % str(p1))
+        monkeypatch.setenv("PYTHONPATH", str(pytester.path))
+        child = pytester.spawn_pytest("--pdbcls=custom_pdb:CustomPdb %s" % str(p1))
 
         child.expect("__init__")
         child.expect("custom set_trace>")
@@ -887,33 +903,22 @@ class TestPDB:
 
 
 class TestDebuggingBreakpoints:
-    def test_supports_breakpoint_module_global(self):
-        """
-        Test that supports breakpoint global marks on Python 3.7+ and not on
-        CPython 3.5, 2.7
-        """
-        if sys.version_info >= (3, 7):
-            assert SUPPORTS_BREAKPOINT_BUILTIN is True
-        if sys.version_info.major == 3 and sys.version_info.minor == 5:
-            assert SUPPORTS_BREAKPOINT_BUILTIN is False  # type: ignore[comparison-overlap]
-
-    @pytest.mark.skipif(
-        not SUPPORTS_BREAKPOINT_BUILTIN, reason="Requires breakpoint() builtin"
-    )
     @pytest.mark.parametrize("arg", ["--pdb", ""])
-    def test_sys_breakpointhook_configure_and_unconfigure(self, testdir, arg):
+    def test_sys_breakpointhook_configure_and_unconfigure(
+        self, pytester: Pytester, arg: str
+    ) -> None:
         """
         Test that sys.breakpointhook is set to the custom Pdb class once configured, test that
         hook is reset to system value once pytest has been unconfigured
         """
-        testdir.makeconftest(
+        pytester.makeconftest(
             """
             import sys
             from pytest import hookimpl
             from _pytest.debugging import pytestPDB
 
             def pytest_configure(config):
-                config._cleanup.append(check_restored)
+                config.add_cleanup(check_restored)
 
             def check_restored():
                 assert sys.breakpointhook == sys.__breakpointhook__
@@ -922,37 +927,33 @@ class TestDebuggingBreakpoints:
                 assert sys.breakpointhook == pytestPDB.set_trace
         """
         )
-        testdir.makepyfile(
+        pytester.makepyfile(
             """
             def test_nothing(): pass
         """
         )
         args = (arg,) if arg else ()
-        result = testdir.runpytest_subprocess(*args)
+        result = pytester.runpytest_subprocess(*args)
         result.stdout.fnmatch_lines(["*1 passed in *"])
 
-    @pytest.mark.skipif(
-        not SUPPORTS_BREAKPOINT_BUILTIN, reason="Requires breakpoint() builtin"
-    )
-    def test_pdb_custom_cls(self, testdir, custom_debugger_hook):
-        p1 = testdir.makepyfile(
+    def test_pdb_custom_cls(self, pytester: Pytester, custom_debugger_hook) -> None:
+        p1 = pytester.makepyfile(
             """
             def test_nothing():
                 breakpoint()
         """
         )
-        result = testdir.runpytest_inprocess(
+        result = pytester.runpytest_inprocess(
             "--pdb", "--pdbcls=_pytest:_CustomDebugger", p1
         )
         result.stdout.fnmatch_lines(["*CustomDebugger*", "*1 passed*"])
         assert custom_debugger_hook == ["init", "set_trace"]
 
     @pytest.mark.parametrize("arg", ["--pdb", ""])
-    @pytest.mark.skipif(
-        not SUPPORTS_BREAKPOINT_BUILTIN, reason="Requires breakpoint() builtin"
-    )
-    def test_environ_custom_class(self, testdir, custom_debugger_hook, arg):
-        testdir.makeconftest(
+    def test_environ_custom_class(
+        self, pytester: Pytester, custom_debugger_hook, arg: str
+    ) -> None:
+        pytester.makeconftest(
             """
             import os
             import sys
@@ -960,7 +961,7 @@ class TestDebuggingBreakpoints:
             os.environ['PYTHONBREAKPOINT'] = '_pytest._CustomDebugger.set_trace'
 
             def pytest_configure(config):
-                config._cleanup.append(check_restored)
+                config.add_cleanup(check_restored)
 
             def check_restored():
                 assert sys.breakpointhook == sys.__breakpointhook__
@@ -970,30 +971,27 @@ class TestDebuggingBreakpoints:
                 assert sys.breakpointhook is _pytest._CustomDebugger.set_trace
         """
         )
-        testdir.makepyfile(
+        pytester.makepyfile(
             """
             def test_nothing(): pass
         """
         )
         args = (arg,) if arg else ()
-        result = testdir.runpytest_subprocess(*args)
+        result = pytester.runpytest_subprocess(*args)
         result.stdout.fnmatch_lines(["*1 passed in *"])
 
-    @pytest.mark.skipif(
-        not SUPPORTS_BREAKPOINT_BUILTIN, reason="Requires breakpoint() builtin"
-    )
     @pytest.mark.skipif(
         not _ENVIRON_PYTHONBREAKPOINT == "",
         reason="Requires breakpoint() default value",
     )
-    def test_sys_breakpoint_interception(self, testdir):
-        p1 = testdir.makepyfile(
+    def test_sys_breakpoint_interception(self, pytester: Pytester) -> None:
+        p1 = pytester.makepyfile(
             """
             def test_1():
                 breakpoint()
         """
         )
-        child = testdir.spawn_pytest(str(p1))
+        child = pytester.spawn_pytest(str(p1))
         child.expect("test_1")
         child.expect("Pdb")
         child.sendline("quit")
@@ -1002,11 +1000,8 @@ class TestDebuggingBreakpoints:
         assert "reading from stdin while output" not in rest
         TestPDB.flush(child)
 
-    @pytest.mark.skipif(
-        not SUPPORTS_BREAKPOINT_BUILTIN, reason="Requires breakpoint() builtin"
-    )
-    def test_pdb_not_altered(self, testdir):
-        p1 = testdir.makepyfile(
+    def test_pdb_not_altered(self, pytester: Pytester) -> None:
+        p1 = pytester.makepyfile(
             """
             import pdb
             def test_1():
@@ -1014,7 +1009,7 @@ class TestDebuggingBreakpoints:
                 assert 0
         """
         )
-        child = testdir.spawn_pytest(str(p1))
+        child = pytester.spawn_pytest(str(p1))
         child.expect("test_1")
         child.expect("Pdb")
         child.sendline("c")
@@ -1025,8 +1020,8 @@ class TestDebuggingBreakpoints:
 
 
 class TestTraceOption:
-    def test_trace_sets_breakpoint(self, testdir):
-        p1 = testdir.makepyfile(
+    def test_trace_sets_breakpoint(self, pytester: Pytester) -> None:
+        p1 = pytester.makepyfile(
             """
             def test_1():
                 assert True
@@ -1038,7 +1033,7 @@ class TestTraceOption:
                 pass
             """
         )
-        child = testdir.spawn_pytest("--trace " + str(p1))
+        child = pytester.spawn_pytest("--trace " + str(p1))
         child.expect("test_1")
         child.expect("Pdb")
         child.sendline("c")
@@ -1056,8 +1051,10 @@ class TestTraceOption:
         assert "Exit: Quitting debugger" not in child.before.decode("utf8")
         TestPDB.flush(child)
 
-    def test_trace_with_parametrize_handles_shared_fixtureinfo(self, testdir):
-        p1 = testdir.makepyfile(
+    def test_trace_with_parametrize_handles_shared_fixtureinfo(
+        self, pytester: Pytester
+    ) -> None:
+        p1 = pytester.makepyfile(
             """
             import pytest
             @pytest.mark.parametrize('myparam', [1,2])
@@ -1075,7 +1072,7 @@ class TestTraceOption:
                 assert request.function.__name__ == "test_func_kw"
             """
         )
-        child = testdir.spawn_pytest("--trace " + str(p1))
+        child = pytester.spawn_pytest("--trace " + str(p1))
         for func, argname in [
             ("test_1", "myparam"),
             ("test_func", "func"),
@@ -1085,12 +1082,12 @@ class TestTraceOption:
             child.expect_exact(func)
             child.expect_exact("Pdb")
             child.sendline("args")
-            child.expect_exact("{} = 1\r\n".format(argname))
+            child.expect_exact(f"{argname} = 1\r\n")
             child.expect_exact("Pdb")
             child.sendline("c")
             child.expect_exact("Pdb")
             child.sendline("args")
-            child.expect_exact("{} = 2\r\n".format(argname))
+            child.expect_exact(f"{argname} = 2\r\n")
             child.expect_exact("Pdb")
             child.sendline("c")
             child.expect_exact("> PDB continue (IO-capturing resumed) >")
@@ -1102,16 +1099,16 @@ class TestTraceOption:
         TestPDB.flush(child)
 
 
-def test_trace_after_runpytest(testdir):
+def test_trace_after_runpytest(pytester: Pytester) -> None:
     """Test that debugging's pytest_configure is re-entrant."""
-    p1 = testdir.makepyfile(
+    p1 = pytester.makepyfile(
         """
         from _pytest.debugging import pytestPDB
 
-        def test_outer(testdir):
+        def test_outer(pytester) -> None:
             assert len(pytestPDB._saved) == 1
 
-            testdir.makepyfile(
+            pytester.makepyfile(
                 \"""
                 from _pytest.debugging import pytestPDB
 
@@ -1122,20 +1119,20 @@ def test_trace_after_runpytest(testdir):
                 \"""
             )
 
-            result = testdir.runpytest("-s", "-k", "test_inner")
+            result = pytester.runpytest("-s", "-k", "test_inner")
             assert result.ret == 0
 
             assert len(pytestPDB._saved) == 1
     """
     )
-    result = testdir.runpytest_subprocess("-s", "-p", "pytester", str(p1))
+    result = pytester.runpytest_subprocess("-s", "-p", "pytester", str(p1))
     result.stdout.fnmatch_lines(["test_inner_end"])
     assert result.ret == 0
 
 
-def test_quit_with_swallowed_SystemExit(testdir):
+def test_quit_with_swallowed_SystemExit(pytester: Pytester) -> None:
     """Test that debugging's pytest_configure is re-entrant."""
-    p1 = testdir.makepyfile(
+    p1 = pytester.makepyfile(
         """
         def call_pdb_set_trace():
             __import__('pdb').set_trace()
@@ -1152,7 +1149,7 @@ def test_quit_with_swallowed_SystemExit(testdir):
             pass
     """
     )
-    child = testdir.spawn_pytest(str(p1))
+    child = pytester.spawn_pytest(str(p1))
     child.expect("Pdb")
     child.sendline("q")
     child.expect_exact("Exit: Quitting debugger")
@@ -1162,9 +1159,9 @@ def test_quit_with_swallowed_SystemExit(testdir):
 
 
 @pytest.mark.parametrize("fixture", ("capfd", "capsys"))
-def test_pdb_suspends_fixture_capturing(testdir, fixture):
+def test_pdb_suspends_fixture_capturing(pytester: Pytester, fixture: str) -> None:
     """Using "-s" with pytest should suspend/resume fixture capturing."""
-    p1 = testdir.makepyfile(
+    p1 = pytester.makepyfile(
         """
         def test_inner({fixture}):
             import sys
@@ -1185,7 +1182,7 @@ def test_pdb_suspends_fixture_capturing(testdir, fixture):
         )
     )
 
-    child = testdir.spawn_pytest(str(p1) + " -s")
+    child = pytester.spawn_pytest(str(p1) + " -s")
 
     child.expect("Pdb")
     before = child.before.decode("utf8")
@@ -1210,9 +1207,9 @@ def test_pdb_suspends_fixture_capturing(testdir, fixture):
     assert "> PDB continue (IO-capturing resumed for fixture %s) >" % (fixture) in rest
 
 
-def test_pdbcls_via_local_module(testdir):
+def test_pdbcls_via_local_module(pytester: Pytester) -> None:
     """It should be imported in pytest_configure or later only."""
-    p1 = testdir.makepyfile(
+    p1 = pytester.makepyfile(
         """
         def test():
             print("before_set_trace")
@@ -1228,7 +1225,7 @@ def test_pdbcls_via_local_module(testdir):
                     print("runcall_called", args, kwds)
         """,
     )
-    result = testdir.runpytest(
+    result = pytester.runpytest(
         str(p1), "--pdbcls=really.invalid:Value", syspathinsert=True
     )
     result.stdout.fnmatch_lines(
@@ -1239,24 +1236,24 @@ def test_pdbcls_via_local_module(testdir):
     )
     assert result.ret == 1
 
-    result = testdir.runpytest(
+    result = pytester.runpytest(
         str(p1), "--pdbcls=mypdb:Wrapped.MyPdb", syspathinsert=True
     )
     assert result.ret == 0
     result.stdout.fnmatch_lines(["*set_trace_called*", "* 1 passed in *"])
 
     # Ensure that it also works with --trace.
-    result = testdir.runpytest(
+    result = pytester.runpytest(
         str(p1), "--pdbcls=mypdb:Wrapped.MyPdb", "--trace", syspathinsert=True
     )
     assert result.ret == 0
     result.stdout.fnmatch_lines(["*runcall_called*", "* 1 passed in *"])
 
 
-def test_raises_bdbquit_with_eoferror(testdir):
+def test_raises_bdbquit_with_eoferror(pytester: Pytester) -> None:
     """It is not guaranteed that DontReadFromInput's read is called."""
 
-    p1 = testdir.makepyfile(
+    p1 = pytester.makepyfile(
         """
         def input_without_read(*args, **kwargs):
             raise EOFError()
@@ -1267,13 +1264,13 @@ def test_raises_bdbquit_with_eoferror(testdir):
             __import__('pdb').set_trace()
         """
     )
-    result = testdir.runpytest(str(p1))
+    result = pytester.runpytest(str(p1))
     result.stdout.fnmatch_lines(["E *BdbQuit", "*= 1 failed in*"])
     assert result.ret == 1
 
 
-def test_pdb_wrapper_class_is_reused(testdir):
-    p1 = testdir.makepyfile(
+def test_pdb_wrapper_class_is_reused(pytester: Pytester) -> None:
+    p1 = pytester.makepyfile(
         """
         def test():
             __import__("pdb").set_trace()
@@ -1295,7 +1292,7 @@ def test_pdb_wrapper_class_is_reused(testdir):
                 print("set_trace_called", args)
         """,
     )
-    result = testdir.runpytest(str(p1), "--pdbcls=mypdb:MyPdb", syspathinsert=True)
+    result = pytester.runpytest(str(p1), "--pdbcls=mypdb:MyPdb", syspathinsert=True)
     assert result.ret == 0
     result.stdout.fnmatch_lines(
         ["*set_trace_called*", "*set_trace_called*", "* 1 passed in *"]
